﻿using System.Collections.Generic;
using Vintagestory.GameContent;
using Vintagestory.API.Common;
using Vintagestory.API.Server;
using Vintagestory.API.Util;

namespace Vintagestory.ServerMods
{
    internal class HauntedChunks : ModSystem
    {
        private ICoreServerAPI serverApi;
        private Dictionary<IServerChunk, int> hauntedChunks;

        public override void StartServerSide(ICoreServerAPI api)
        {
            serverApi = api;

            api.Event.SaveGameLoaded += OnSaveGameLoading;
            api.Event.GameWorldSave += OnSaveGameSaving;

            api.Event.PlayerDeath += OnPlayerDeath;
            api.Event.DidUseBlock += OnUseBlock;
        }

        private void OnSaveGameLoading()
        {
            hauntedChunks = new Dictionary<IServerChunk, int>();
        }

        private void OnSaveGameSaving()
        {
            foreach (KeyValuePair<IServerChunk, int> chunk in hauntedChunks)
            {
                chunk.Key.SetServerModdata("haunting", SerializerUtil.Serialize(chunk.Value));
            }
        }

        private void OnPlayerDeath(IServerPlayer byPlayer, DamageSource damageSource)
        {
            IServerChunk chunk = serverApi.WorldManager.GetChunk(byPlayer.Entity.ServerPos.AsBlockPos);

            if (!hauntedChunks.ContainsKey(chunk))
            {
                AddChunkToDictionary(chunk);
            }

            hauntedChunks[chunk]++;
        }

        private void OnUseBlock(IServerPlayer byPlayer, BlockSelection blockSel)
        {
            if (serverApi.World.BlockAccessor.GetBlock(blockSel.Position).GetType() != typeof(BlockBed))
            {
                return;
            }

            IServerChunk chunk = serverApi.WorldManager.GetChunk(blockSel.Position);

            if (!hauntedChunks.ContainsKey(chunk))
            {
                AddChunkToDictionary(chunk);
            }

            int haunting = hauntedChunks[chunk];
            if (haunting > 0)
            {
                byPlayer.Entity.ReceiveDamage(new DamageSource()
                {
                    Source = EnumDamageSource.Void,
                    Type = EnumDamageType.BluntAttack
                }, haunting);
            }
        }

        private void AddChunkToDictionary(IServerChunk chunk)
        {
            byte[] data = chunk.GetServerModdata("haunting");
            int haunting = data == null ? 0 : SerializerUtil.Deserialize<int>(data);

            hauntedChunks.Add(chunk, haunting);
        }
    }
}